// File: src/lib.rs
// Author: chmod777
// Creation Date: 2021-2-23
// License: AGPLv3

#[cfg(test)]
mod tests;

pub mod config;
mod handlers;

use html5ever::local_name;
use kuchiki::traits::*;
use kuchiki::{NodeData, NodeRef};

use config::Config;
use handlers::*;

pub struct State {
	pub gemtext: String,
	pub config: Config,
}
impl Default for State {
	fn default() -> Self {
		State {
			gemtext: String::new(),
			config: Config::default(),
		}
	}
}

fn handle_node_recurisive(state: &mut State, current: &NodeRef) {
	if let Ok("Recursion Handled") = handle_node(state, current) {
	} else {
		for ref node in current.children() {
			handle_node_recurisive(state, node);
		}
	}
}

fn handle_node(state: &mut State, node: &NodeRef) -> Result<&'static str, String> {
	match node.data() {
		NodeData::Element(element_data) => match element_data.name.local {
			local_name!("h1") => handle_header_node(state, 1, node),
			local_name!("h2") => handle_header_node(state, 2, node),
			local_name!("h3") => handle_header_node(state, 3, node),
			local_name!("h4") => handle_header_node(state, 4, node),
			local_name!("h5") => handle_header_node(state, 5, node),
			local_name!("h6") => handle_header_node(state, 6, node),
			local_name!("p") => handle_paragraph_node(state, node),
			local_name!("a") => handle_link_node(state, element_data, node),
			local_name!("img") => handle_image_node(state, element_data),
			local_name!("li") => handle_list_node(state, node),
			local_name!("br") => handle_break_node(state, node),
			_ => Ok(""),
		},
		_ => Ok(""),
	}
}

pub fn convert(html_src: &str) -> String {
	let document = kuchiki::parse_html().one(html_src);

	let body_selector = "body";
	let body = document.select(body_selector).unwrap();

	let mut state = State::default();
	state.config.convert_mydomain_links_to_gmi = Some(String::from("mydomain"));

	for node_data_ref in body {
		let current = node_data_ref.as_node();
		handle_node_recurisive(&mut state, current);
	}

	state.gemtext
}
